/*
   Bounding Sphere Collision Detection.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to perform collision detection using
   bounding spheres.  This is simlar to the bounding boxes in terms of its purpose but
   bounding sphere are a little different to set up.  When it comes to which is better
   that depends on the object.  Bounding spheres are faster than boxes so if speed is an
   issue and accuracy is not then this is the demo for you.
*/


// Demo: Bounding Sphere Collision Detection.


#define WIN32_LEAN_AND_MEAN
#define VC_LEANMEAN

#include<d3dx9.h>
#include"CCamera.h"
#include"CVector.h"


#define D3DFVF_D3DVertex (D3DFVF_XYZ | D3DFVF_TEX2)


// Function Prototypes...
bool InitializeDirect3D(HWND hwnd, bool fullscreen);
bool InitializeObject();
void SetPerpective();
void RenderScene();
void ShutdownDirect3D();
void GetInput();


// DirectX objects.
LPDIRECT3D9 Direct3D_Object = NULL;
LPDIRECT3DDEVICE9 D3D_Device = NULL;


LPDIRECT3DVERTEXBUFFER9 Object_Buffer = NULL;
LPDIRECT3DVERTEXBUFFER9 TexCoord_Buffer = NULL;
LPDIRECT3DVERTEXBUFFER9 Floor_Buffer = NULL;
LPDIRECT3DVERTEXBUFFER9 floorTexCoord_Buffer = NULL;
LPDIRECT3DTEXTURE9 GroundImage = NULL;
LPDIRECT3DTEXTURE9 Decal = NULL;
LPDIRECT3DVERTEXDECLARATION9 VertexDeclaration = NULL;

int gWidth = 0;
int gHeight = 0;

// Camera object used to move around the 3D scene.
CCamera Camera;

// Here is the structure for our bounding sphere.
struct BoundingSphere
{
   CVector4 Center;
   double radius;
};

// This will hold the cube's bounding sphere data.
BoundingSphere CubesSphere;

// This is the center of the cube.  I filled in the values for the center so we don't
// have to calculate it.  Normally you would calculate it but to keep things simple I
// just calculated it myself and coded it in here.
CVector4 Center(0, 0, -0.1f);

// This will hold the cube we will draw to the screen.
CVector4 Cube[24];
CTexCoord texCoord[24];

// Ground data.
CVector4 floorData[4];
CTexCoord floorTexCoord[4];


void CalculateBoundingSphere()
{
   float Distance, maxDistance = 0.0f;
   CubesSphere.Center = Center;

   // Loop through all of the cube's data to find the radius.  First we get the distance of
   // each point and store it in Distance.  If Distance is greater than the last max
   // distance in (maxDistance) then maxDistance is set to Distance.  Keep going
   // until you go through all  points.
   for(int i = 0; i < 24; i++)
      {
         Distance = ((Cube[i].x - Center.x) * (Cube[i].x - Center.x)) +
                    ((Cube[i].y - Center.y) * (Cube[i].y - Center.y)) +
                    ((Cube[i].z - Center.z) * (Cube[i].z - Center.z));

		   if(Distance > maxDistance)
            maxDistance = Distance;
      }

   // Once we have the max distance then then radius is the square root of it.
   CubesSphere.radius = sqrt(maxDistance);
}


bool OnCollision(CVector4 point1, CVector4 point2, double sR1)
{
   // The distance between the two spheres.
   CVector4 Intersect;

   // Calculate the distance between them.  First get the vector like so...
   Intersect = point1 - point2;
	
   // Now the distance is the square root of the vector.  If the distance is less
   // then the two radius added together then we have collision.
   if(sqrt(Intersect.x * Intersect.x + Intersect.y * Intersect.y +
           Intersect.z * Intersect.z) < sR1)
      return true;

   return false;
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   int width, height;

    switch(message)
       {
		   case WM_KEYUP:
            switch(wParam)
               { 
		            case VK_ESCAPE:
                     PostQuitMessage(0);
				         break;
               }
            break;

         case WM_SIZE:                       // re-size message.
            height = HIWORD(lParam);         // This gets the height of the window.
            width = LOWORD(lParam);          // This gets the width of the window.

            if(height==0)                    // we don't want it to be possible for a
               {                             // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            gWidth = width;
            gHeight = height;

            return 0;
            break;

         case WM_CLOSE:
         case WM_DESTROY:
            PostQuitMessage(0);
            break;

            default:
               break;
       }

    return DefWindowProc(hwnd, message, wParam, lParam);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                                           // Message object.
   HWND hwnd;                                         // Handle to the window.
   WNDCLASSEX windowClass;                            // Window class object.
   bool done = false;                                 // True then exit.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);           // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;       // style of the window.
   windowClass.lpfnWndProc = WndProc;                 // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                        // Extra class information.
   windowClass.cbWndExtra = 0;                        // Extra window information.
   windowClass.hInstance = hInstance;                 // Handle of application Instance.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);// Window Icon.
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW); // Load mouse cursor.
   windowClass.hbrBackground = NULL;                  // Background color.
   windowClass.lpszMenuName = NULL;                   // Menu.
   windowClass.lpszClassName = "UGPClass";            // Name of the window class.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);// Minimized window icon.

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create the window.
   hwnd = CreateWindowEx(NULL,                                       // The extended style.
                         "UGPClass",                                 // Window class.
                         "Bounding Sphere - by The Programming Ace", // Window name.
                          WS_OVERLAPPEDWINDOW | WS_VISIBLE |         // Window style.
                          WS_SYSMENU |WS_CLIPCHILDREN |              // Window style.
                          WS_CLIPSIBLINGS,                           // Window style.
                          100, 100,                                  // X, Y coords.
                          640, 480,                                  // Window size.
                          NULL,                                      // Handle to parent window.
                          NULL,                                      // Menu.
                          hInstance,                                 // Handle to app instance.
                          NULL);                                     // Pointer to window.

   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW);    // Show the window.
   UpdateWindow(hwnd);           // Update its display.

   done = false;                 // false = run program, true means stop.

   // Initialize Direct3D.  If fails then we don't want to run the application.
   if(!InitializeDirect3D(hwnd, false)) done = true;

   // Application loop.
   while(!done)
      {
         if(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     done = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }


    // Here we shutdown Direct 3D.
	ShutdownDirect3D();

   // Here we unregister the window class with the OS.
   UnregisterClass("UGPClass", windowClass.hInstance);

   return (int)msg.wParam;
}


bool InitializeDirect3D(HWND hwnd, bool fullscreen)
{
   // This object will allow us to set the display mode of the screen.
   D3DDISPLAYMODE DisplayMode;

   // This will allow us to set the parameters of the screen.
   D3DPRESENT_PARAMETERS Present_Parameters;

   // This is used to get the capabilities of the hardware.
   D3DCAPS9 D3DCaps;

   // It is always a good idea to clear out memory in object although not necessary.
   ZeroMemory(&Present_Parameters, sizeof(Present_Parameters));

   // Create the Direct3D object to get everything started.
   Direct3D_Object = Direct3DCreate9(D3D_SDK_VERSION);

   // Error checking.  Make sure that it was successful.
   if(Direct3D_Object == NULL)
      {
         MessageBox(NULL, "Error, couldn't initialize DirectX!?!",
                    "Error!", MB_OK);
         return false;
      }

   // This function will get the display mode of the device and place it in DisplayMode.
   if(FAILED(Direct3D_Object->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &DisplayMode)))
      {
         MessageBox(NULL, "Error setting the display mode.", "Error!", MB_OK);
         return false;
      }

   // Get the capabilities of the hardware.
	if(FAILED(Direct3D_Object->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &D3DCaps)))
		{
		   return false;
	   }

   // Test which is supported, hardware or software vertex processing.
	DWORD VertexProcessing = 0;

	if(D3DCaps.VertexProcessingCaps != 0)
		VertexProcessing |= D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		VertexProcessing |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;

   // Here we are setting the applications parameters...
   if(fullscreen)
      {
         Present_Parameters.Windowed = FALSE;               // Window mode (fullscreen).
         Present_Parameters.BackBufferWidth = 640;
         Present_Parameters.BackBufferHeight = 480;
      }
   else
      Present_Parameters.Windowed = TRUE;                   // Window mode (not fullscreen).
   Present_Parameters.SwapEffect = D3DSWAPEFFECT_DISCARD;   // Dealing with animation (see doc).
   Present_Parameters.BackBufferFormat = DisplayMode.Format;// Render to the area of the screen.
   Present_Parameters.BackBufferCount = 1;                  // Number of back buffers.
   Present_Parameters.EnableAutoDepthStencil = TRUE;        // Check documentation.
   Present_Parameters.AutoDepthStencilFormat = D3DFMT_D16;  // Check documentation.


   // Now we must create the rendering device.
   if(FAILED(Direct3D_Object->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                           VertexProcessing,
                                           &Present_Parameters, &D3D_Device)))
      {
         MessageBox(NULL, "CreateDevice() failed!  Make sure you have DirectX 9.",
                    "Error!", MB_OK);
         return false;
      }


   // One last check to be sure.
   if(D3D_Device == NULL)
      {
         MessageBox(NULL, "D3D_Device is equal to NULL!?!", "Error!", MB_OK);
         return false;
      }

   // SetRenderState() will set a parameter for the rendering.  Here we turn off lighting.
   D3D_Device->SetRenderState(D3DRS_LIGHTING, FALSE);

   // Disable culling.  This will render the front and back of the triangles.
	D3D_Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

   // Enable depth testing.
   D3D_Device->SetRenderState(D3DRS_ZENABLE, TRUE);

   // Here we set up the object this demo will render (for example a triangle).
   if(!InitializeObject()) return false;

   // Here we set the perpective.
   SetPerpective();

   // Set the image states to get a good quality image.
	D3D_Device->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	D3D_Device->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

   return true;
}


bool InitializeObject()
{
   // Here we will initialize the box's data.  We will create each side of the cube.
   // Notice we dont do the top and bottom, I didn't think it was worth taking up more space.
   // Front.
   Cube[0].x = -0.1f; Cube[0].y = -0.1f; Cube[0].z = 0.0f;
   texCoord[0].tu = 1.0f; texCoord[0].tv = 1.0f;
   Cube[1].x = 0.1f; Cube[1].y = -0.1f; Cube[1].z = 0.0f;
   texCoord[1].tu = 0.0f; texCoord[1].tv = 1.0f;
   Cube[2].x = 0.1f;  Cube[2].y = 0.1f; Cube[2].z = 0.0f;
   texCoord[2].tu = 0.0f; texCoord[2].tv = 0.0f;
   Cube[3].x = 0.1f;  Cube[3].y = 0.1f; Cube[3].z = 0.0f;
   texCoord[3].tu = 0.0f; texCoord[3].tv = 0.0f;
   Cube[4].x = -0.1f;  Cube[4].y = 0.1f; Cube[4].z = 0.0f;
   texCoord[4].tu = 1.0f; texCoord[4].tv = 0.0f;
   Cube[5].x = -0.1f;  Cube[5].y = -0.1f; Cube[5].z = 0.0f;
   texCoord[5].tu = 1.0f; texCoord[5].tv = 1.0f;

   // Back.
   Cube[6].x = -0.1f; Cube[6].y = -0.1f; Cube[6].z = -0.2f;
   texCoord[6].tu = 1.0f; texCoord[6].tv = 1.0f;
   Cube[7].x = 0.1f; Cube[7].y = -0.1f; Cube[7].z = -0.2f;
   texCoord[7].tu = 0.0f; texCoord[7].tv = 1.0f;
   Cube[8].x = 0.1f;  Cube[8].y = 0.1f; Cube[8].z = -0.2f;
   texCoord[8].tu = 0.0f; texCoord[8].tv = 0.0f;
   Cube[9].x = 0.1f;  Cube[9].y = 0.1f; Cube[9].z = -0.2f;
   texCoord[9].tu = 0.0f; texCoord[9].tv = 0.0f;
   Cube[10].x = -0.1f;  Cube[10].y = 0.1f; Cube[10].z = -0.2f;
   texCoord[10].tu = 1.0f; texCoord[10].tv = 0.0f;
   Cube[11].x = -0.1f;  Cube[11].y = -0.1f; Cube[11].z = -0.2f;
   texCoord[11].tu = 1.0f; texCoord[11].tv = 1.0f;

   // Right.
   Cube[12].x = -0.1f; Cube[12].y = -0.1f; Cube[12].z = 0.0f;
   texCoord[12].tu = 1.0f; texCoord[12].tv = 1.0f;
   Cube[13].x = -0.1f; Cube[13].y = -0.1f; Cube[13].z = -0.2f;
   texCoord[13].tu = 0.0f; texCoord[13].tv = 1.0f;
   Cube[14].x = -0.1f;  Cube[14].y = 0.1f; Cube[14].z = -0.2f;
   texCoord[14].tu = 0.0f; texCoord[14].tv = 0.0f;
   Cube[15].x = -0.1f;  Cube[15].y = 0.1f; Cube[15].z = -0.2f;
   texCoord[15].tu = 0.0f; texCoord[15].tv = 0.0f;
   Cube[16].x = -0.1f;  Cube[16].y = 0.1f; Cube[16].z = 0.0f;
   texCoord[16].tu = 1.0f; texCoord[16].tv = 0.0f;
   Cube[17].x = -0.1f;  Cube[17].y = -0.1f; Cube[17].z = 0.0f;
   texCoord[17].tu = 1.0f; texCoord[17].tv = 1.0f;

   // Left.
   Cube[18].x = 0.1f; Cube[18].y = -0.1f; Cube[18].z = 0.0f;
   texCoord[18].tu = 1.0f; texCoord[18].tv = 1.0f;
   Cube[19].x = 0.1f; Cube[19].y = -0.1f; Cube[19].z = -0.2f;
   texCoord[19].tu = 0.0f; texCoord[19].tv = 1.0f;
   Cube[20].x = 0.1f;  Cube[20].y = 0.1f; Cube[20].z = -0.2f;
   texCoord[20].tu = 0.0f; texCoord[20].tv = 0.0f;
   Cube[21].x = 0.1f;  Cube[21].y = 0.1f; Cube[21].z = -0.2f;
   texCoord[21].tu = 0.0f; texCoord[21].tv = 0.0f;
   Cube[22].x = 0.1f;  Cube[22].y = 0.1f; Cube[22].z = 0.0f;
   texCoord[22].tu = 1.0f; texCoord[22].tv = 0.0f;
   Cube[23].x = 0.1f;  Cube[23].y = -0.1f; Cube[23].z = 0.0f;
   texCoord[23].tu = 1.0f; texCoord[23].tv = 1.0f;

   // Now that we have the cube initialized we will now calculate the bounding box's data.
   CalculateBoundingSphere();

   // Create the vertex buffer that will hold the triangle.
	if(FAILED(D3D_Device->CreateVertexBuffer(sizeof(Cube),
                                            0, D3DFVF_D3DVertex,
                                            D3DPOOL_DEFAULT, &Object_Buffer, NULL)))
      {
         return false;
      }
	
   // Pointer to the buffer.
   CVector4* Vertices;

   // Lock the buffer we can write to it.
   if(FAILED(Object_Buffer->Lock(0, sizeof(Cube), (void**)&Vertices, 0)))
      return false;

   // Here we copy the triangle's data into the vertex buffer.
   memcpy(Vertices, Cube, sizeof(Cube));

   // Unlock when your done coping data into the buffer.
   Object_Buffer->Unlock();

   // Create the vertex buffer that will hold the texture coordinates.
	if(FAILED(D3D_Device->CreateVertexBuffer(sizeof(texCoord),
                                            0, 0,
                                            D3DPOOL_DEFAULT, &TexCoord_Buffer, NULL)))
      {
         return false;
      }
	
   // Pointer to the buffer.
   CTexCoord* TexC;

   // Lock the buffer we can write to it.
   if(FAILED(TexCoord_Buffer->Lock(0, sizeof(texCoord), (void**)&TexC, 0)))
      return false;

   // Copy the tex coords into the buffer for the tex coords.
   memcpy(TexC, texCoord, sizeof(texCoord));

   // Unlock when your done coping data into the buffer.
   TexCoord_Buffer->Unlock();

   
   // Create the ground.
   floorData[0] = CVector4(-4.0f, -0.1f, -4.0f);
   floorData[1] = CVector4(-4.0f, -0.1f, 4.0f);
   floorData[2] = CVector4(4.0f, -0.1f, -4.0f);
   floorData[3] = CVector4(4.0f, -0.1f, 4.0f);
   floorTexCoord[0].tu = 0.0f; floorTexCoord[0].tv = 1.0f;
   floorTexCoord[1].tu = 0.0f; floorTexCoord[1].tv = 0.0f;
   floorTexCoord[2].tu = 1.0f; floorTexCoord[2].tu = 1.0f;
   floorTexCoord[3].tu = 1.0f; floorTexCoord[3].tu = 0.0f;


   // Create the vertex buffer that will hold the triangle.
	if(FAILED(D3D_Device->CreateVertexBuffer(sizeof(floorData),
                                            0, D3DFVF_D3DVertex,
                                            D3DPOOL_DEFAULT, &Floor_Buffer, NULL)))
      {
         return false;
      }
	
   // Lock the buffer we can write to it.
   if(FAILED(Floor_Buffer->Lock(0, sizeof(floorData), (void**)&Vertices, 0)))
      return false;

   // Here we copy the triangle's data into the vertex buffer.
   memcpy(Vertices, floorData, sizeof(floorData));

   // Unlock when your done coping data into the buffer.
   Floor_Buffer->Unlock();

   // Create the vertex buffer that will hold the texture coordinates.
	if(FAILED(D3D_Device->CreateVertexBuffer(sizeof(floorTexCoord),
                                            0, 0,
                                            D3DPOOL_DEFAULT, &floorTexCoord_Buffer, NULL)))
      {
         return false;
      }
	
   // Pointer to the buffer.
   CTexCoord* fTexC;

   // Lock the buffer we can write to it.
   if(FAILED(floorTexCoord_Buffer->Lock(0, sizeof(floorTexCoord), (void**)&fTexC, 0)))
      return false;

   // Copy the tex coords into the buffer for the tex coords.
   memcpy(fTexC, floorTexCoord, sizeof(floorTexCoord));

   // Unlock when your done coping data into the buffer.
   floorTexCoord_Buffer->Unlock();


   // Here we describe to Direct3D how a vertex is to be described.
   D3DVERTEXELEMENT9 Declaration[] = 
	   {
	      {0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		   {1, 0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
		   D3DDECL_END()
	   };

   // Create the vertex declaration so DirectX knows how our vertex is made up.
   D3D_Device->CreateVertexDeclaration(Declaration, &VertexDeclaration);


   // Here we load the texture into the texture object.
   if(D3DXCreateTextureFromFile(D3D_Device, "ground.bmp", &GroundImage) != D3D_OK)
      return false;

   if(D3DXCreateTextureFromFile(D3D_Device, "decal.tga", &Decal) != D3D_OK)
      return false;


   // Here we set the camera's default position.
   Camera.SetCamera(0.0f, 0.0f, 2.0f, 0.0f, 0.0f, 1.85f, 0.0f, 1.0f, 0.0f);

   return true;
}


void SetPerpective()
{
   // This matrix will do nothing more but tell Direct3D how far back it can see
   // and how close it can see.
   D3DXMATRIXA16 Projection;

   // Set the info into the matrix.
   D3DXMatrixPerspectiveFovLH(&Projection, 45.0f, (float)(gWidth/gHeight), 0.1f, 500.0f);

   // Set the projection.
   D3D_Device->SetTransform(D3DTS_PROJECTION, &Projection);
}


void RenderScene()
{
   // Clear the screen.
   D3D_Device->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                     D3DCOLOR_XRGB(127, 127, 255), 1.0f, 0);
    
   // Tell Direct 3D we are now going to start drawing.
   D3D_Device->BeginScene();

      // Used to hold view matrix.
      D3DXMATRIXA16 View;

      // Get the input from the user so we can move the camera either foward or back.
      GetInput();

      // Set the camera.
      D3DXVECTOR3 Eye(Camera.xPos, Camera.yPos, Camera.zPos);
      D3DXVECTOR3 LookAt(Camera.xView, Camera.yView, Camera.zView);
      D3DXVECTOR3 Up(Camera.xUp, Camera.yUp, Camera.zUp);

      // Set all the values this matrix will need to know (for the camera).
      D3DXMatrixLookAtLH(&View, &Eye, &LookAt, &Up);

      // Move the "camera" to the position and values we set above.
      D3D_Device->SetTransform(D3DTS_VIEW, &View);

      // Draw the cube.
      D3D_Device->SetTexture(0, Decal);
      D3D_Device->SetStreamSource(0, Object_Buffer, 0, sizeof(CVector4));
      D3D_Device->SetStreamSource(1, TexCoord_Buffer, 0, sizeof(CTexCoord));
      D3D_Device->SetVertexDeclaration(VertexDeclaration);
      D3D_Device->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 8);


      // Draw the floor now.
      D3D_Device->SetTexture(0, GroundImage);
      D3D_Device->SetStreamSource(0, Floor_Buffer, 0, sizeof(CVector4));
      D3D_Device->SetStreamSource(1, floorTexCoord_Buffer, 0, sizeof(CTexCoord));
      D3D_Device->SetVertexDeclaration(VertexDeclaration);
      D3D_Device->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
    
   // We are done drawing.
   D3D_Device->EndScene();
    
   // Swap the old frame with the new one.
   D3D_Device->Present(NULL, NULL, NULL, NULL);
}


void ShutdownDirect3D()
{
   // Here we release the Direct3D objects.
   if(GroundImage != NULL) 
      {
         GroundImage->Release();
         GroundImage = NULL;
      }

   if(Decal != NULL) 
      {
         Decal->Release();
         Decal = NULL;
      }

   if(D3D_Device != NULL)
      {
         D3D_Device->Release();
         D3D_Device = NULL;
      }

   if(Direct3D_Object != NULL)
      {
         Direct3D_Object->Release();
         Direct3D_Object = NULL;
      }

   if(Object_Buffer != NULL)
      {
	      Object_Buffer->Release();
         Object_Buffer = NULL;
      }

   if(TexCoord_Buffer != NULL)
      {
	      TexCoord_Buffer->Release();
         TexCoord_Buffer = NULL;
      }

   if(Floor_Buffer != NULL)
      {
	      Floor_Buffer->Release();
         Floor_Buffer = NULL;
      }

   if(floorTexCoord_Buffer != NULL)
      {
	      floorTexCoord_Buffer->Release();
         floorTexCoord_Buffer = NULL;
      }

   if(VertexDeclaration != NULL)
      {
         VertexDeclaration->Release();
         VertexDeclaration = NULL;
      }
}


void GetInput()
{
   // Update the time for time based movement.
   Camera.CalculateTime();

   // When we check for collision we send in the center position of the first object
   // and the camera's position.  Just like in the bounding box demo we use the view
   // position to test when we are moving forward, the postion for everything else.
   if(GetKeyState(VK_UP) & 0x80)
      {
         if(!OnCollision(CubesSphere.Center,
                         CVector4(Camera.xView, Camera.yView, Camera.zView),
                         CubesSphere.radius))
		      {
               Camera.MoveCamera(UP);                    // Move the camera foward.
            }
      }

   // This is the same of the up key except that we will use the position
   // to test for collision.
   if(GetKeyState(VK_DOWN) & 0x80)
      {
         if(!OnCollision(CubesSphere.Center,
                         CVector4(Camera.xPos, Camera.yPos, Camera.zPos),
                         CubesSphere.radius))
		      {
               Camera.MoveCamera(DOWN);                  // Move the camera back.
            }
      }

   // Now we can rotate the camera without testing because it rotates around in place.
   if(GetKeyState(VK_LEFT) & 0x80)
      Camera.RotateCamera(LEFT, 0.0f, 1.0f, 0.0f);   // Rotate the camera left.

   if(GetKeyState(VK_RIGHT) & 0x80)
      Camera.RotateCamera(RIGHT, 0.0f, 1.0f, 0.0f);  // Rotate the camera right.

   // Now we test strafing left and right the same way as if we pressed the down key.
   if(GetKeyState('A') & 0x80 || GetKeyState('a') & 0x80)
      {
         if(!OnCollision(CubesSphere.Center,
                         CVector4(Camera.xPos, Camera.yPos, Camera.zPos),
                         CubesSphere.radius))
		      {
               Camera.StrafeCam(STRAFE_LEFT);            // Strafe the camera left.
            }
      }

   // Same as the left.
   if(GetKeyState('D') & 0x80 || GetKeyState('d') & 0x80)
      {
         if(!OnCollision(CubesSphere.Center,
                         CVector4(Camera.xPos, Camera.yPos, Camera.zPos),
                         CubesSphere.radius))
		      {
               Camera.StrafeCam(STRAFE_RIGHT);           // Side step right.
            }
      }
}


// Recap:

// Oh come on that waste that bad was it?  Bounding spheres are faster than bounding
// cubes in terms for calculating for collision.  The problem with sphere is that
// they is a lot of unused space around the object that will make collision detection
// not as accurate (but it is fast).  I've played a lot of games in my day that must
// have used this method because after testing this demo the first time I got that
// familar feeling.  As time goes on I will be creating more collision detection
// demos until I feel we have something that is good to have in all situations.
// Until then have fun experimenting.


// Copyright September 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com