/*
   Class Name:

      CCamera.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class represents a camera in a 3D scene.
*/


#include"CCamera.h"
#include"windows.h"


CCamera::CCamera()
{
   // Initialize variables...
   xPos = yPos = zPos = 0;
   xView = yView = zView = 0;
   xUp = yUp = zUp = 0;
   xStrafe = yStrafe = zStrafe = 0;
   frameInterval = 0.0f;
}


void CCamera::SetCamera(float x, float y, float z,
                        float xv, float yv, float zv,
                        float xu, float yu, float zu)
{
   // Here we set the camera to the values sent in to us.  This is mostly used to set up a
   // default position.
   xPos = x;   yPos = y;   zPos = z;
   xView = xv; yView = yv; zView = zv;
   xUp = xu;   yUp = yu;   zUp = zu;
}


void CCamera::MoveCamera(float direction)
{
   // Moving the camera requires a little more then adding 1 to the z or subracting 1.
   // First we need to get the direction at which we are looking.
   float xLookDirection = 0, yLookDirection = 0, zLookDirection = 0;

   // The look direction is the view (where we are looking) minus the position (where we are).
   xLookDirection = xView - xPos;
	yLookDirection = yView - yPos;
	zLookDirection = zView - zPos;

   // Normalize the direction.
   float dp = 1 /(float)sqrt(xLookDirection * xLookDirection + yLookDirection * yLookDirection +
                             zLookDirection * zLookDirection);
   xLookDirection *= dp;
   yLookDirection *= dp;
   zLookDirection *= dp;

   // Call UpdateCamera to move our camera in the direction we want.
   UpdateCamera(xLookDirection, yLookDirection, zLookDirection, direction);

   // NOTE if the camera moves too slow try increasing the UP and DOWN values in the header.
   // The camera is frame based so it will move differently for you (unless you have the same
   // PC specs as me).
}


void CCamera::UpdateCamera(float xDir, float yDir, float zDir, float dir)
{
   // We use our time based movement when we update the camera.
   dir = dir * frameInterval;

   // Move the camera on the X and Z axis.  Notice I commented out the yPos and yView
   // updates.  This is because without them we can keep the camera on the ground in
   // the simple camera tutorials without having to alter them afterwards.
	xPos += xDir * dir;
   //yPos += yDir * dir;
	zPos += zDir * dir;

   // Move the view along with the position
	xView += xDir * dir;
   //yView += yDir * dir;
	zView += zDir * dir;
}


void CCamera::StrafeCam(float direction)
{
   CalculateStrafe();                           // Calculate the straft direction.
   UpdateCamera(xStrafe, yStrafe, zStrafe, direction);   // Then add it to the camera's position.
}


void CCamera::CalculateStrafe()
{
   float xDir = 0, yDir = 0, zDir = 0;
   float xCross = 0, yCross = 0, zCross = 0;

   // Strafing is just like moving the camera forward and backward.  First we will get the
   // direction we are looking.
   xDir = xView - xPos;
	yDir = yView - yPos;
	zDir = zView - zPos;

   // Normalize the direction.
   float dp = 1 /(float)sqrt(xDir * xDir + yDir * yDir + zDir * zDir);
   xDir *= dp;
   yDir *= dp;
   zDir *= dp;

   // Now if we were to call UpdateCamera() we will be moving the camera foward or backwards.
   // We don't want that here.  We want to strafe.  To do so we have to get the cross product
   // of our direction and Up direction view.  The up was set in SetCamera to be 1 positive
   // y.  That is because anything positive on the y is considered up.  After we get the
   // cross product we can save it to the strafe variables so that can be added to the
   // camera using UpdateCamera().

   // Get the cross product of the direction we are looking and the up direction.
   xCross = (yDir * zUp) - (zDir * yUp);
   yCross = (zDir * xUp) - (xDir * zUp);
   zCross = (xDir * yUp) - (yDir * xUp);

   // Save our strafe (cross product) values in xStrafe, yStrafe, and zStrafe.
   xStrafe = xCross;
   yStrafe = yCross;
   zStrafe = zCross;
}


void CCamera::RotateCamera(float AngleDir, float xSpeed, float ySpeed, float zSpeed)
{
	CQuaternion qRotation, qView, qNewView;

   // Create the rotation quaternion based on the axis we are rotating on.
   qRotation.Rotatef(AngleDir, xSpeed, ySpeed, zSpeed);

   // Create the view quaternion.  This will be the direction of the view and position.
   qView.x = xView - xPos;
   qView.y = yView - yPos;
   qView.z = zView - zPos;
   qView.w = 0;

   // Create the resulting quaternion by multiplying the rotation quat by the view quat
   // then multiplying that by the conjugate of the rotation quat.
   qNewView = ((qRotation * qView) * qRotation.Conjugate());

   // Update the view information by adding the position to the resulting quaternion.
   xView = xPos + qNewView.x;
   yView = yPos + qNewView.y;
   zView = zPos + qNewView.z;
}


void CCamera::RotateByMouse(int mousePosX, int mousePosY, int midX, int midY)
{
	float yDirection = 0.0f;         // Direction angle.
	float yRotation = 0.0f;          // Rotation angle.

	// If the mouseX and mouseY are at the middle of the screen then we can't rotate the view.
	if((mousePosX == midX) && (mousePosY == midY))
      return;

	// Next we get the direction of each axis.  We divide by a value to get a smaller value back.
	yDirection = (float)((midX - mousePosX)) / 20.0f;		
	yRotation = (float)((midY - mousePosY)) / 20.0f;		

	// We use curentRotX to help use keep the camera from rotating too far in either direction.
	currentRotationAngle -= yRotation;

	// Stop the camera from going to high...
	if(currentRotationAngle > 30.0f)
      {
		   currentRotationAngle = 30.0f;
         return;
      }

	// Stop the camera from going to low...
	if(currentRotationAngle < -30.0f)
		{
         currentRotationAngle = -30.0f;
         return;
      }

   // Next we get the axis which is a perpendicular vector of the view direction and up values.
   // We use the cross product of that to get the axis then we normalize it.
   float xAxis = 0, yAxis = 0, zAxis = 0;
   float xDir = 0, yDir = 0, zDir = 0;

   // Get the Direction of the view.
   xDir = xView - xPos;
	yDir = yView - yPos;
	zDir = zView - zPos;

   // Get the cross product of the direction and the up.
   xAxis = (yDir * zUp) - (zDir * yUp);
   yAxis = (zDir * xUp) - (xDir * zUp);
   zAxis = (xDir * yUp) - (yDir * xUp);

   // Normalize it.
   float len = 1 /(float)sqrt(xAxis * xAxis + yAxis * yAxis + zAxis * zAxis);
   xAxis *= len;
   yAxis *= len;
   zAxis *= len;

   // Rotate the camera.
   RotateCamera(yRotation, xAxis, yAxis, zAxis);
   RotateCamera(-yDirection, 0, 1, 0);
}


void CCamera::CalculateTime()
{
   static float lastFrameTime = 0.0f;// Last frame's time.

   // Get the time in milliseconds (we convert to second by * 0.001).
   float currentTime = timeGetTime() * 0.001f;

   // Calculate the frame interval by the current time minus the last frame's time.
   frameInterval = currentTime - lastFrameTime;
   lastFrameTime = currentTime;
}


// Copyright September 2003
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com